package com.yfd.platform.modules.experimentalData.service.impl;

import cn.hutool.core.collection.CollUtil;


import java.nio.file.*;
import java.security.DigestInputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;


import com.amazonaws.services.s3.model.ObjectMetadata;
import com.amazonaws.services.s3.model.S3Object;
import com.amazonaws.services.s3.model.S3ObjectInputStream;
import com.amazonaws.util.IOUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qiniu.storage.model.FileInfo;
import com.yfd.platform.config.ResponseResult;
import com.yfd.platform.modules.experimentalData.domain.*;
import com.yfd.platform.modules.experimentalData.mapper.TsFilesMapper;
import com.yfd.platform.modules.experimentalData.mapper.TsNodesMapper;
import com.yfd.platform.modules.experimentalData.service.ITsFilesService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.modules.storage.chain.FileChain;
import com.yfd.platform.modules.storage.context.StorageSourceContext;
import com.yfd.platform.modules.storage.mapper.StorageSourceConfigMapper;
import com.yfd.platform.modules.storage.mapper.StorageSourceMapper;
import com.yfd.platform.modules.storage.model.entity.StorageSourceConfig;
import com.yfd.platform.modules.storage.model.enums.FileTypeEnum;
import com.yfd.platform.modules.storage.model.request.BatchDeleteRequest;
import com.yfd.platform.modules.storage.model.request.NewFolderRequest;
import com.yfd.platform.modules.storage.model.request.RenameFileRequest;
import com.yfd.platform.modules.storage.model.result.FileItemResult;
import com.yfd.platform.modules.storage.service.StorageSourceService;
import com.yfd.platform.modules.storage.service.base.AbstractBaseFileService;
import com.yfd.platform.system.domain.LoginUser;
import com.yfd.platform.utils.StringUtils;
import io.netty.channel.ChannelInboundHandlerAdapter;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.*;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

/**
 * <p>
 * 试验任务文档表 服务实现类
 * </p>
 *
 * @author LiMengNan
 * @since 2025-01-21
 */
@Service
public class TsFilesServiceImpl extends ServiceImpl<TsFilesMapper, TsFiles> implements ITsFilesService {
    private static final Logger LOGGER = LoggerFactory.getLogger(ChannelInboundHandlerAdapter.class);


    //试验任务文档表 Mapper
    @Resource
    private TsFilesMapper tsFilesMapper;

    //试验任务节点表 Mapper
    @Resource
    private TsNodesMapper tsNodesMapper;


    //数据源Mapper
    @Resource
    private StorageSourceMapper storageSourceMapper;


    @Resource
    private StorageSourceContext storageSourceContext;

    @Resource
    private StorageSourceService storageSourceService;

    @Resource
    private FileChain fileChain;

    @Resource
    private StorageSourceConfigMapper storageSourceConfigMapper;


    /**********************************
     * 用途说明: 分页查询试验数据管理-文档内容
     * 参数说明
     *id            id
     * fileName      文件名称
     * startDate     (开始日期)
     * endDate      （结束日期）
     * keywords      关键字
     *nodeId         节点ID
     *taskId          任务ID
     *isFile          文件夹、文件区分
     * pageNum 当前页
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回分页查询结果
     ***********************************/
    @Override
    public Page<TsFiles> getTsFilesPage(String id, String fileName, String startDate, String endDate, String keywords, String nodeId, String taskId, String fileName1, String childNode, Page<TsFiles> page) throws Exception {


        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();

        // 固定条件过滤
        if (StringUtils.isNotBlank(taskId)) queryWrapper.eq("task_id", taskId);
        if (StringUtils.isNotBlank(nodeId)) queryWrapper.eq("node_id", nodeId);

        // 模糊搜索
        if (StringUtils.isNotBlank(fileName)) queryWrapper.like("file_name", fileName);
        if (StringUtils.isNotBlank(keywords)) queryWrapper.like("keywords", keywords);

        // 时间范围过滤
        if (StringUtils.isNotBlank(startDate)) queryWrapper.ge("upload_time", startDate);
        if (StringUtils.isNotBlank(endDate)) queryWrapper.le("upload_time", endDate);
        boolean hasSearchCondition = StringUtils.isNotBlank(fileName) || StringUtils.isNotBlank(keywords) || StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate);

        // 处理层级关系条件
        if (StringUtils.isNotBlank(id)) {
            // 有ID时：根据childNode决定是否递归子节点   如果filename
            if (childNode != null && childNode.equals("0") && hasSearchCondition) {
                List<String> childIds = getAllChildIds(id);
                queryWrapper.in("parent_id", childIds);
            } else {
                // 其他情况：childNode=1 或 无搜索条件时 按当前层级查询
                queryWrapper.eq("parent_id", id);
            }
        } else {
            //如果说ID是空 说明是顶级节点
            if (childNode != null) {
                if (childNode.equals("0")) {
                    if (hasSearchCondition) {
                        // 递归获取根目录下所有子节点ID（包含多级）
                        List<String> allChildIds = getAllChildIds("00"); // 注意这里传入字符串"00"
                        allChildIds.add("00"); // 包含根目录的直接子节点
                        queryWrapper.in("parent_id", allChildIds);
                    } else {
                        // 无搜索条件时，仅查询根目录的直接子节点
                        queryWrapper.eq("parent_id", "00");
                    }
                } else if (childNode.equals("1")) {
                    // 始终只查根目录的直接子节点
                    queryWrapper.eq("parent_id", "00");
                }
            }

//            // 无ID时：childNode=1则查根目录，childNode=0查全表
//            if (childNode != null && childNode.equals("1")) {
//                queryWrapper.eq("parent_id", "00");
//            }
        }
        //
        queryWrapper.isNotNull("work_path");
        //排序
        queryWrapper.orderByDesc("is_file");//时间
//        queryWrapper.orderByDesc("upload_time");//时间
        //分页查询
        Page<TsFiles> tsFilesPage = tsFilesMapper.selectPage(page, queryWrapper);
        if (tsFilesPage == null) {
            tsFilesPage = new Page<>();
            tsFilesPage.setRecords(new ArrayList<>()); // 确保 records 被初始化
        }
        List<TsFiles> records = tsFilesPage.getRecords();
        for (TsFiles tsFiles : records) {
            //如果是文件夹 跳过循环
            if (tsFiles.getIsFile().equals("FOLDER")) {
                if(tsFiles.getUpdateTime() == null){
                    tsFiles.setUpdateTime(tsFiles.getUploadTime());
                }
                continue;
            }
            String workPath = tsFiles.getWorkPath();
            String fileNameData = tsFiles.getFileName();
            //主要是用于文件路径加名称
            String path = workPath + fileNameData;
            //准备获取文件的信息
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("local");
            FileItemResult fileItemResult = fileService.getFileItem(path);
            tsFiles.setUrl(fileItemResult.getUrl());
            //如果是压缩文件 类型就给zip
            boolean isValid = hasValidExtension(fileItemResult.getName());
            if (isValid) {
                tsFiles.setType("ZIP");
            } else {
                tsFiles.setType(fileItemResult.getType().getValue());
            }
            if(tsFiles.getUpdateTime() == null){
                tsFiles.setUpdateTime(tsFiles.getUploadTime());
            }
        }
        tsFilesPage.setRecords(records); // 同步到 tsFilesPage
        System.out.println("Updated records: " + records);
        return tsFilesPage;

    }

    public boolean hasValidExtension(String name) {
        if (name == null) {
            return false; // 如果传入的文件名为空，返回 false
        }
        // 判断文件名后缀是否是 .zip 或 .tar
        return name.endsWith(".zip") || name.endsWith(".tar");
    }

    // 递归获取所有子节点ID（包含隔代子节点）
    private List<String> getAllChildIds(String parentId) {
        List<String> childIds = new ArrayList<>();
        List<TsFiles> children = tsFilesMapper.selectList(
                new QueryWrapper<TsFiles>().select("id").eq("parent_id", parentId).eq("is_file", "FOLDER") // 只遍历文件夹节点
        );
        for (TsFiles child : children) {
            childIds.add(child.getId());
            childIds.addAll(getAllChildIds(child.getId()));
        }
        return childIds;
    }

    /**********************************
     * 用途说明:   查询实验数据管理文件夹
     * 参数说明
     * pageNum 当前页
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回分页查询结果
     ***********************************/
    @Override
    public List<TsFiles> getsListTsFiles(String id, String path) {
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_file", "FOLDER");
        if (StringUtils.isNotBlank(id)) {
            queryWrapper.eq("parent_id", id);
        }
        if (StringUtils.isNotBlank(path)) {
            queryWrapper.eq("work_path", path);
        }
        queryWrapper.isNotNull("work_path");
        List<TsFiles> tsFiles = tsFilesMapper.selectList(queryWrapper);
        return tsFiles;
    }


//    /**********************************
//     * 用途说明: 分页查询试验数据管理-文件夹下内容
//     * 参数说明
//     * id            id
//     * pageNum 当前页
//     * 返回值说明: com.yfd.platform.config.ResponseResult 返回分页查询结果
//     ***********************************/
//    @Override
//    public Page<TsFiles> getsubFilepage(String id, Page<TsFiles> page) {
//        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("parent_id", id);
//        //分页查询
//        Page<TsFiles> subFilepage = tsFilesMapper.selectPage(page, queryWrapper);
//        if (subFilepage == null) {
//            subFilepage = new Page<>();
//            subFilepage.setRecords(new ArrayList<>()); // 确保 records 被初始化
//        }
//        List<TsFiles> records = subFilepage.getRecords();
//        for (TsFiles tsFiles : records) {
//            //如果是文件夹 跳过循环
//            if (tsFiles.getIsFile().equals("FOLDER")) {
//                continue;
//            }
//            String workPath = tsFiles.getWorkPath();
//            String fileNameData = tsFiles.getFileName();
//            //主要是用于文件路径加名称
//            String path = workPath + fileNameData;
//            //准备获取文件的信息
//            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("local");
//            FileItemResult fileItemResult = fileService.getFileItem(path);
//            tsFiles.setUrl(fileItemResult.getUrl());
//            //如果是压缩文件 类型就给zip
//            boolean isValid = hasValidExtension(fileItemResult.getName());
//            if (isValid) {
//                tsFiles.setType("ZIP");
//            } else {
//                tsFiles.setType(fileItemResult.getType().getValue());
//            }
//        }
//        subFilepage.setRecords(records); // 同步到 tsFilesPage
//        return subFilepage;
//    }


    /***********************************
     * 用途说明：新增试验数据管理-文档内容
     * 参数说明
     * TsFiles     文档内容
     * 返回值说明:  com.yfd.platform.config.ResponseResult 返回新增成功或者失败
     ***********************************/
    @Override
    public Boolean addTsFiles(TsFiles tsFiles) {

        Boolean value = false;
        //文件名称和大小   因为支持多个上传所以用，分隔
        List<String> names = Arrays.asList(tsFiles.getFileName().split(","));
        List<String> sizes = Arrays.asList(tsFiles.getFileSize().split(","));
        //上传人是当前登录人
        UsernamePasswordAuthenticationToken authentication =
                (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();

        // 数据校验
        if (names.size() != sizes.size()) {
            LOGGER.error("文件名称和文件大小的列表长度不一致");
            return false;
        }

        List<TsFiles> filesToSave = new ArrayList<>();
        // 设置当前时间
        LocalDateTime now = LocalDateTime.now();
        // 转换为 Timestamp
        Timestamp currentTime = Timestamp.valueOf(now);
        tsFiles.setUploadTime(currentTime);

        for (int i = 0; i < names.size(); i++) {
            String name = names.get(i).trim();
            String sizeStr = sizes.get(i).trim();
            // 校验文件大小是否可以转换成数值
            try {
                TsFiles files1 = new TsFiles();
                files1.setTaskId(tsFiles.getTaskId());
                files1.setIsFile(tsFiles.getIsFile());
                files1.setParentId(tsFiles.getParentId());
                files1.setNodeId(tsFiles.getNodeId());
                files1.setWorkPath(tsFiles.getWorkPath());
                files1.setBackupPath(tsFiles.getBackupPath());
                files1.setKeywords(tsFiles.getKeywords());
                files1.setDescription(tsFiles.getDescription());
                files1.setUploadTime(tsFiles.getUploadTime());
                files1.setUploader(loginuser.getUsername());
                files1.setFileName(name);
                files1.setFileSize(sizeStr);
                filesToSave.add(files1);
            } catch (NumberFormatException e) {
                LOGGER.error("文件大小必须是有效的数字");
            }
        }
        if (filesToSave.size() > 0) {
            //循环新增
            for (TsFiles filess : filesToSave) {
                int valueAdded = tsFilesMapper.insert(filess);
                if (valueAdded == 1) {
                    value = true;
                } else {
                    value = false;
                }
            }
        }
        return value;
    }


    /***********************************
     * 用途说明：新增试验数据管理-文件夹
     * 参数说明
     * TsFiles     文档内容
     * 返回值说明:  com.yfd.platform.config.ResponseResult 返回新增成功或者失败
     ***********************************/
    @Override
    @Transactional(rollbackFor = Exception.class)// 添加事务注解，遇到异常时回滚
    public ResponseResult addTsFile(TsFiles tsFiles) {

        //上传人是当前登录人
        UsernamePasswordAuthenticationToken authentication =
                (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();
        // 设置当前时间
        LocalDateTime now = LocalDateTime.now();
        // 转换为 Timestamp
        Timestamp currentTime = Timestamp.valueOf(now);
        tsFiles.setUploadTime(currentTime);
        tsFiles.setUploader(loginuser.getUsername());
        tsFiles.setFileSize("0");
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("node_id", tsFiles.getNodeId());
        queryWrapper.eq("task_id", tsFiles.getTaskId());
        queryWrapper.eq("parent_id", tsFiles.getParentId());
        queryWrapper.eq("file_name", tsFiles.getFileName());
        int count = tsFilesMapper.selectCount(queryWrapper);
        // 大于0说明 区域名称重复
        if (count > 0) {
            return ResponseResult.error("文件夹名称已存在！");
        }
        //本地创建文件夹
        NewFolderRequest newFolderRequest = new NewFolderRequest();
        newFolderRequest.setStorageKey("local");
        newFolderRequest.setPath(tsFiles.getWorkPath());
        newFolderRequest.setPassword(null);
        newFolderRequest.setName(tsFiles.getFileName());

        AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
        boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
        if (flag) {
            int valueAdded = tsFilesMapper.insert(tsFiles);
            if (valueAdded == 1) {
                LOGGER.info("表结构和本地都创建文件夹成功");
                return ResponseResult.success();
            } else {
                LOGGER.error("本地创建文件夹成功，表结构新增失败");
                return ResponseResult.error();
            }
        } else {
            LOGGER.error("本地创建文件夹失败");
            return ResponseResult.error();
        }
    }


    /**********************************
     * 用途说明: 修改试验数据管理-文档内容
     * 参数说明
     * TsFiles     文档内容
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回修改成功或者失败
     ***********************************/
    @Override
    @Transactional(rollbackFor = Exception.class)// 添加事务注解，遇到异常时回滚
    public ResponseResult updateTsFiles(TsFiles tsFiles) {
        // 修改之前查询表中的文件名是否修改，如果发生变动先修改 minio 然后再修改表结构
        TsFiles filesData = tsFilesMapper.selectById(tsFiles.getId());

        // 判断文件名是否修改  不一致需要修改
        if (!tsFiles.getFileName().equals(filesData.getFileName())) {

            //校验文件夹会不会重命名
            if ("FOLDER".equals(tsFiles.getIsFile())) {
                //如果是文件夹 递归修改它下面的所有文件的路径
                QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("node_id", tsFiles.getNodeId());
                queryWrapper.eq("task_id", tsFiles.getTaskId());
                queryWrapper.eq("parent_id", tsFiles.getParentId());
                queryWrapper.eq("file_name", tsFiles.getFileName());
                int count = tsFilesMapper.selectCount(queryWrapper);
                // 大于0说明 区域名称重复
                if (count > 0) {
                    return ResponseResult.error("文件夹名称已存在！");
                }

                //查询数据源
                RenameFileRequest renameFileRequest = new RenameFileRequest();
                renameFileRequest.setName(filesData.getFileName());
                renameFileRequest.setNewName(tsFiles.getFileName());
                renameFileRequest.setPassword("");
                renameFileRequest.setPath(filesData.getWorkPath());
                renameFileRequest.setStorageKey("local");
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(renameFileRequest.getStorageKey());
                Boolean flag = fileService.renameFile(renameFileRequest.getPath(), renameFileRequest.getName(), renameFileRequest.getNewName());
                //如果是true  说明至少有一个修改了文件夹
                if (flag) {
                    //递归下面所有的文件文件夹  路径都需要修改
                    List<TsFiles> resultList = new ArrayList<>();

                    // 获取父路径（去除自身）
                    String parentPath = filesData.getWorkPath();
                    String[] pathParts = parentPath.split("/");
                    int subscript = pathParts.length - 1;
                    //ID  旧的文件名   新的文件名
                    querySubFilesAndUpdatePaths(resultList, tsFiles.getId(), filesData.getFileName(), tsFiles.getFileName(), subscript);
                    // 修改数据库
                    int valueUpdate = tsFilesMapper.updateById(tsFiles);
                    if (valueUpdate == 1) {
                        LOGGER.error("\"local和minio修改成功，表结构修改成功\"");
                        return ResponseResult.success();
                    } else {
                        LOGGER.error("local和minio修改成功，表结构修改失败");
                        return ResponseResult.error();
                    }
                } else {
                    LOGGER.error("local和minio修改失败");
                    return ResponseResult.error();
                }
            } else {
                //如果是文件   直接修改就可以
                QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("node_id", tsFiles.getNodeId());
                queryWrapper.eq("task_id", tsFiles.getTaskId());
                queryWrapper.eq("parent_id", tsFiles.getParentId());
                queryWrapper.eq("file_name", tsFiles.getFileName());
                int count = tsFilesMapper.selectCount(queryWrapper);
                // 大于0说明 区域名称重复
                if (count > 0) {
                    return ResponseResult.error("文件名称已存在！");
                }
                //查询数据源
                RenameFileRequest renameFileRequest = new RenameFileRequest();
                renameFileRequest.setName(filesData.getFileName());
                renameFileRequest.setNewName(tsFiles.getFileName());
                renameFileRequest.setPassword("");
                renameFileRequest.setPath(filesData.getWorkPath());
                renameFileRequest.setStorageKey("local");
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(renameFileRequest.getStorageKey());
                Boolean flag = fileService.renameFile(renameFileRequest.getPath(), renameFileRequest.getName(), renameFileRequest.getNewName());
                //如果是true  说明至少有一个修改了文件夹
                if (flag) {
                    // 修改数据库
                    int valueUpdate = tsFilesMapper.updateById(tsFiles);
                    if (valueUpdate == 1) {
                        LOGGER.error("\"local和minio修改成功，表结构修改成功\"");
                        return ResponseResult.success();
                    } else {
                        LOGGER.error("local和minio修改成功，表结构修改失败");
                        return ResponseResult.error();
                    }
                } else {
                    LOGGER.error("local和minio修改失败");
                    return ResponseResult.error();
                }
            }
        } else {
            // 如果文件名没有修改，仅更新数据库
            int valueUpdate = tsFilesMapper.updateById(tsFiles);
            if (valueUpdate == 1) {
                return ResponseResult.success();
            } else {
                return ResponseResult.error();
            }
        }
    }


    private void querySubFilesAndUpdatePaths(List<TsFiles> resultList, String id, String oldName, String newName, int subscript) {
        // 构造 QueryWrapper 查询当前文件夹下的文件
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", id);
        // 查询当前文件夹下的所有子项
        List<TsFiles> subFiles = tsFilesMapper.selectList(queryWrapper);
        // 确保 subFiles 不为空
        if (subFiles == null || subFiles.isEmpty()) {
            return;
        }

        // 将查询到的结果添加到结果列表
        resultList.addAll(subFiles);

        // 遍历每个子项，更新路径并递归查询其子项
        for (TsFiles subFile : subFiles) {
            // 获取原始路径
            String oldWorkPath = subFile.getWorkPath();
            // 替换路径中的 oldName 为 newName，仅替换路径数组中指定下标的部分
            String newWorkPath = updatePathAtSubscript(oldWorkPath, oldName, newName, subscript);
            // 更新数据库中的路径
            subFile.setWorkPath(newWorkPath);
            tsFilesMapper.updateById(subFile);
            // 递归查询每个子项的子文件
            if ("FOLDER".equals(subFile.getIsFile())) {
                querySubFilesAndUpdatePaths(resultList, subFile.getId(), oldName, newName, subscript);
            }
            querySubFilesAndUpdatePaths(resultList, subFile.getId(), oldName, newName, subscript);
        }
    }

    // 辅助方法，更新路径中特定下标的部分
    private String updatePathAtSubscript(String oldPath, String oldName, String newName, int subscript) {
        // 过滤空值
        List<String> pathParts = Arrays.stream(oldPath.split("/"))
                .filter(s -> !s.isEmpty())
                .collect(Collectors.toList());

        // 确保路径数组下标有效
        if (pathParts.size() > subscript) {
            pathParts.set(subscript, pathParts.get(subscript).replace(oldName, newName));
        }

        // 拼接成新的路径
        return "/" + String.join("/", pathParts) + "/";
    }

    /**********************************
     * 用途说明: 批量删除试验数据管理-文档内容
     * 参数说明  ids 文档内容id数组
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回批量删除成功或失败
     ***********************************/
    @Override
    @Transactional(rollbackFor = Exception.class)// 添加事务注解，遇到异常时回滚
    public String deleteTsFilesByIds(List<String> dataset) {
        List<TsFiles> filesList = tsFilesMapper.selectBatchIds(dataset);

        int LocalSuccessCount = 0, LocalFailCount = 0, Localtotal = CollUtil.size(dataset);
        //Todo  最直接的办法   循环出来 一条一条删除
        for (TsFiles files : filesList) {

            //判断是文件还是文件夹
            if ("FOLDER".equals(files.getIsFile())) {
                //如果是文件夹
                List<BatchDeleteRequest.DeleteItem> deleteItemList = new ArrayList<>();
                BatchDeleteRequest.DeleteItem deleteItemData = new BatchDeleteRequest.DeleteItem();
                deleteItemData.setName(files.getFileName());
                deleteItemData.setPassword("");
                deleteItemData.setPath(files.getWorkPath());
                deleteItemData.setType(FileTypeEnum.FOLDER);
                deleteItemList.add(deleteItemData);
                //首先通过ID集合查询所有的内容  然后放到List<DeleteItem> deleteItems里面  放好以后删除数据库  然后删除minio
                BatchDeleteRequest batchDeleteRequest = new BatchDeleteRequest();
                batchDeleteRequest.setDeleteItems(deleteItemList);
                batchDeleteRequest.setStorageKey("local");

                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(batchDeleteRequest.getStorageKey());
                List<BatchDeleteRequest.DeleteItem> deleteItems = batchDeleteRequest.getDeleteItems();

                int deleteSuccessCount = 0, deleteFailCount = 0, totalCount = CollUtil.size(deleteItems);
                for (BatchDeleteRequest.DeleteItem deleteItem : deleteItems) {

                    boolean flag = false;
                    try {
                        if (deleteItem.getType() == FileTypeEnum.FILE) {
                            flag = fileService.deleteFile(deleteItem.getPath(), deleteItem.getName());
                        } else if (deleteItem.getType() == FileTypeEnum.FOLDER) {
                            flag = fileService.deleteFolder(deleteItem.getPath(), deleteItem.getName());
                        }

                        if (flag) {
                            deleteSuccessCount++;
                        } else {
                            deleteFailCount++;
                        }
                    } catch (Exception e) {
                        LOGGER.error("删除文件/文件夹失败, 文件路径: {}, 文件名称: {}", deleteItem.getPath(), deleteItem.getName(), e);
                        deleteFailCount++;
                    }
                }

                if (totalCount > 1) {
                    //return ResponseResult.success("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                    LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                } else {
                    //return totalCount == deleteSuccessCount ? ResponseResult.success("删除成功") : ResponseResult.error("删除失败");
                    LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                }

                //如果是1 说明成功删除
                if (deleteSuccessCount >= 1) {
                    int valueDelete = deleteFolder(files.getId());
                    if (valueDelete > 0) {
                        LocalSuccessCount = valueDelete;
                        LOGGER.info("表结构成功删除");
                    } else {
                        System.out.println("没有记录被删除");
                    }
                } else {
                    LocalFailCount++;
                }
            } else {
                //如果是文件

                List<BatchDeleteRequest.DeleteItem> deleteItemList = new ArrayList<>();
                BatchDeleteRequest.DeleteItem deleteItemData = new BatchDeleteRequest.DeleteItem();
                deleteItemData.setName(files.getFileName());
                deleteItemData.setPassword("");
                deleteItemData.setPath(files.getWorkPath());
                deleteItemData.setType(FileTypeEnum.FILE);
                deleteItemList.add(deleteItemData);
                //首先通过ID集合查询所有的内容  然后放到List<DeleteItem> deleteItems里面  放好以后删除数据库  然后删除minio
                BatchDeleteRequest batchDeleteRequest = new BatchDeleteRequest();
                batchDeleteRequest.setDeleteItems(deleteItemList);
                batchDeleteRequest.setStorageKey("local");

                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(batchDeleteRequest.getStorageKey());
                List<BatchDeleteRequest.DeleteItem> deleteItems = batchDeleteRequest.getDeleteItems();

                int deleteSuccessCount = 0, deleteFailCount = 0, totalCount = CollUtil.size(deleteItems);
                for (BatchDeleteRequest.DeleteItem deleteItem : deleteItems) {

                    boolean flag = false;
                    try {
                        if (deleteItem.getType() == FileTypeEnum.FILE) {
                            flag = fileService.deleteFile(deleteItem.getPath(), deleteItem.getName());
                        } else if (deleteItem.getType() == FileTypeEnum.FOLDER) {
                            flag = fileService.deleteFolder(deleteItem.getPath(), deleteItem.getName());
                        }

                        if (flag) {
                            deleteSuccessCount++;
                        } else {
                            deleteFailCount++;
                        }
                    } catch (Exception e) {
                        LOGGER.error("删除文件/文件夹失败, 文件路径: {}, 文件名称: {}", deleteItem.getPath(), deleteItem.getName(), e);
                        deleteFailCount++;
                    }
                }

                if (totalCount > 1) {
                    //return ResponseResult.success("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                    LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                } else {
                    //return totalCount == deleteSuccessCount ? ResponseResult.success("删除成功") : ResponseResult.error("删除失败");
                    LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");

                }
                //如果是1 说明成功删除
                if (deleteSuccessCount == 1) {
                    int valueDelete = tsFilesMapper.deleteById(files.getId());
                    if (valueDelete > 0) {
                        LocalSuccessCount++;
                        LOGGER.info("表结构成功删除");
                    } else {
                        System.out.println("没有记录被删除");
                    }
                } else {
                    LocalFailCount++;
                }
            }
        }
        return "Local批量删除 " + Localtotal + " 个, 删除成功 " + LocalSuccessCount + " 个, 失败 " + LocalFailCount + " 个.";
    }

    // 调用删除方法并返回删除数量
    public int deleteFolder(String folderId) {
        return deleteFolderAndSubFiles(folderId);  // 返回删除的记录数量
    }

    // 递归删除文件夹及其子文件，并返回删除的记录数量
    public int deleteFolderAndSubFiles(String folderId) {
        int deletedCount = 0;  // 记录删除的数量

        // 查询该文件夹下的所有子文件
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", folderId); // 根据父ID查询所有子文件
        List<TsFiles> subFiles = tsFilesMapper.selectList(queryWrapper);

        // 递归删除子文件
        for (TsFiles subFile : subFiles) {
            // 递归调用删除子文件的逻辑
            deletedCount += deleteFolderAndSubFiles(subFile.getId());  // 累加删除数量

            // 删除子文件记录（从数据库中删除）
            tsFilesMapper.deleteById(subFile.getId());
            deletedCount++;  // 每删除一个文件，数量加1
        }

        // 删除文件夹本身（根据传入的文件夹 ID 删除）
        tsFilesMapper.deleteById(folderId);
        deletedCount++;  // 删除文件夹本身，数量加1

        return deletedCount;  // 返回删除的数量
    }


    /**********************************
     * 用途说明: 压缩文件夹接口
     * 参数说明  ids 文件id数组
     * 参数说明  compressedFormat 压缩文件格式
     * 参数说明  compressedName 压缩文件名称
     * 参数说明  compressedPath 压缩文件路径
     * 参数说明  covered 是否覆盖 0 覆盖更新updateTime时间 1提示文件存在
     * 参数说明  parentId 父ID
     * 返回值说明: com.yfd.platform.config.ResponseResult
     ***********************************/
    @Override
    public String compressFolder(String ids, String compressedFormat, String compressedName, String compressedPath, String covered, String parentId) throws FileNotFoundException {
        //获取本地根路径
        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);
        String returnResult = "";
        // 创建要压缩的文件夹列表
        List<Path> sourceDirs = new ArrayList<>(); // 修复：使用 ArrayList
        String[] splitIds = ids.split(",");
        // 数组转集合
        List<String> dataset = Arrays.asList(splitIds); // 使用 Arrays.asList 创建不可变列表
        List<TsFiles> filesList = tsFilesMapper.selectBatchIds(dataset);
        // 检查 filesList 是否为空
        if (filesList == null || filesList.isEmpty()) {
            throw new RuntimeException("未找到要压缩的文件或文件夹");
        }
        for (TsFiles tsFiles : filesList) {
            // 检查 tsFiles 的字段是否为空
            if (tsFiles.getWorkPath() == null || tsFiles.getFileName() == null) {
                throw new RuntimeException("文件或文件夹信息不完整");
            }

            // 构建完整路径
            String sourceDir = storageSourceConfig.getValue() + tsFiles.getWorkPath() + tsFiles.getFileName();
            Path path = Paths.get(sourceDir);
            // 检查路径是否存在
            if (!Files.exists(path)) {
                throw new FileNotFoundException("路径不存在: " + path);
            }
            // 添加到压缩列表
            sourceDirs.add(path); // 修复：使用可变集合
        }

        // 规范化压缩包存放路径
        // 如果 compressedPath 是根目录，使用当前工作目录
        Path zipFilePath = Paths.get(storageSourceConfig.getValue() + compressedPath + compressedName + "." + compressedFormat);

        // 检查目标路径是否可写
        if (!Files.isWritable(zipFilePath.getParent())) {
            throw new RuntimeException("目标路径不可写: " + zipFilePath.getParent());
        }

        String zipFileName = compressedName + "." + compressedFormat;

        // 使用 try-with-resources 确保 zipOut 在使用后关闭
        try (ZipArchiveOutputStream zipOut = new ZipArchiveOutputStream(new FileOutputStream(zipFilePath.toFile()))) {
            // 设置压缩级别，可以选择调整
            zipOut.setLevel(ZipArchiveOutputStream.STORED); // 或者使用其他的压缩级别
            // 调用压缩方法
            Boolean value = compressDirectoriesToZip(sourceDirs, zipOut);
            if (value) {
                //表结构增加    nodeId, String TaskId
                TsFiles tsFiles = new TsFiles();
                tsFiles.setTaskId(filesList.get(0).getTaskId());
                tsFiles.setNodeId(filesList.get(0).getNodeId());
                tsFiles.setWorkPath(compressedPath);
                tsFiles.setIsFile("FILE");
                tsFiles.setParentId(parentId);
                tsFiles.setBackupPath("");
                tsFiles.setKeywords("");
                tsFiles.setDescription("");
                tsFiles.setFileName(zipFileName);

                //上传人是当前登录人
                UsernamePasswordAuthenticationToken authentication = (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
                LoginUser loginuser = (LoginUser) authentication.getPrincipal();

                List<TsFiles> filesToSave = new ArrayList<>();
                // 设置当前时间
                LocalDateTime now = LocalDateTime.now();
                // 转换为 Timestamp
                Timestamp currentTime = Timestamp.valueOf(now);
                tsFiles.setUploadTime(currentTime);
                tsFiles.setUploader(loginuser.getUsername());
                // 通过 nodeid  taskid  名称查询  如果存在就不 新增

                QueryWrapper<TsFiles> queryWrapperTsFiles = new QueryWrapper<>();
                queryWrapperTsFiles.eq("node_id", filesList.get(0).getNodeId());
                queryWrapperTsFiles.eq("task_id", filesList.get(0).getTaskId());
                queryWrapperTsFiles.eq("work_path", compressedPath);
                queryWrapperTsFiles.eq("file_name", zipFileName);
                queryWrapperTsFiles.eq("parent_id", filesList.get(0).getParentId());
                TsFiles tsFilesdata = tsFilesMapper.selectOne(queryWrapperTsFiles);
                //covered 是否覆盖 0 覆盖更新updateTime时间 1提示文件存在
                if ("0".equals(covered)) {
                    //如果不为空 更新updateTime时间
                    if (tsFilesdata != null) {
                        tsFilesdata.setUpdateTime(currentTime);
                        tsFilesMapper.updateById(tsFilesdata);
                    } else {
                        //如果为空就新增
                        int valueAdded = tsFilesMapper.insert(tsFiles);
                        returnResult = "压缩成功";
                    }

                } else {
                    // 1提示文件存在   如果不为空返回提示
                    if (tsFilesdata != null) {
                        returnResult = "文件或文件夹已存在！";
                    } else {
                        //如果为空就新增
                        int valueAdded = tsFilesMapper.insert(tsFiles);
                        returnResult = "压缩成功";
                    }
                }
                returnResult = "压缩成功";
            } else {
                returnResult = "压缩失败";
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return returnResult;
    }

    private boolean compressDirectoriesToZip(List<Path> sourceDirs, ZipArchiveOutputStream zipOut) {
        // 用于记录已处理的规范路径，防止循环引用
        Set<Path> processedPaths = new HashSet<>();
        for (Path sourceDir : sourceDirs) {
            // 遍历时传入已处理路径集合
            if (!compressDirectoryToZip(sourceDir, zipOut, "", processedPaths)) {
                return false;
            }
        }
        return true;
    }


    /**
     * 递归压缩目录到ZIP文件
     *
     * @param sourceDir      当前要压缩的目录/文件路径
     * @param zipOut         ZIP输出流
     * @param parentDir      当前路径在ZIP中的父目录（用于构建层级）
     * @param processedPaths 已处理的规范路径集合（用于检测循环引用）
     * @return 压缩成功返回true，否则false
     */
    private boolean compressDirectoryToZip(Path sourceDir, ZipArchiveOutputStream zipOut,
                                           String parentDir, Set<Path> processedPaths) {
        try {
            // --- 防循环处理 Start ---
            // 获取规范路径（解析符号链接），用于检测重复目录
            Path canonicalPath = sourceDir.toRealPath();

            // 如果是目录且已处理过，直接跳过
            if (Files.isDirectory(sourceDir) && processedPaths.contains(canonicalPath)) {
                return true; // 已处理，避免循环
            }
            // 将当前目录加入已处理集合（先添加再处理子项）
            processedPaths.add(canonicalPath);
            // --- 防循环处理 End ---

            File currentFile = sourceDir.toFile();

            // 处理文件（当sourceDirs包含文件时）
            if (!currentFile.isDirectory()) {
                // 直接压缩单个文件
                addFileToZip(currentFile, zipOut, parentDir);
                return true;
            }

            // 获取目录下的文件列表（处理空目录和权限问题）
            File[] files = currentFile.listFiles();
            if (files == null) {
                System.err.println("无法访问目录: " + sourceDir);
                return false; // 目录不可访问
            }

            // 遍历目录内容
            for (File file : files) {
                String entryName = parentDir + file.getName();

                if (file.isDirectory()) {
                    // 创建目录条目（以/结尾）
                    zipOut.putArchiveEntry(new ZipArchiveEntry(entryName + "/"));
                    zipOut.closeArchiveEntry();

                    // 递归处理子目录，传入更新后的父路径和已处理集合
                    if (!compressDirectoryToZip(file.toPath(), zipOut, entryName + "/", processedPaths)) {
                        return false;
                    }
                } else {
                    // 压缩文件
                    addFileToZip(file, zipOut, entryName);
                }
            }
            return true;
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
    }

    /**
     * 将单个文件添加到ZIP流
     *
     * @param file      要添加的文件
     * @param zipOut    ZIP输出流
     * @param entryName 文件在ZIP中的路径
     */
    private void addFileToZip(File file, ZipArchiveOutputStream zipOut, String entryName) throws IOException {
        zipOut.putArchiveEntry(new ZipArchiveEntry(entryName));
        try (FileInputStream fis = new FileInputStream(file)) {
            byte[] buffer = new byte[1024];
            int len;
            while ((len = fis.read(buffer)) > 0) {
                zipOut.write(buffer, 0, len);
            }
        }
        zipOut.closeArchiveEntry();
    }


    /*************************************解压缩*******************************************/

    /**********************************
     * 用途说明: 解压缩接口
     * 参数说明  id  要解压的文件id
     * 参数说明  decompressionPath 解压缩路径
     * 参数说明  parentId 父ID
     * 返回值说明: com.yfd.platform.config.ResponseResult
     ***********************************/
    @Override
    public String decompressionFolder(String id, String decompressionPath, String parentId) {
        // 查询本地文件路径根目录（如 E:\yun）
        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);

        // 获取压缩文件记录
        TsFiles zipFileRecord = tsFilesMapper.selectById(id);
        String localRootDir = storageSourceConfig.getValue() + decompressionPath;

        try {
            // 解压到临时目录
            Path zipFilePath = Paths.get(localRootDir, zipFileRecord.getFileName());
            File unzippedRoot = unzipToTemp(zipFilePath, localRootDir);

            // 递归保存结构到数据库
            saveFolderStructure(unzippedRoot, parentId, decompressionPath, zipFileRecord.getTaskId(), zipFileRecord.getNodeId(), zipFileRecord.getUploader());

            return "解压存储成功";
        } catch (Exception e) {
            throw new RuntimeException("操作失败: " + e.getMessage());
        }
    }

    /**
     * 保存文件夹结构
     *
     * @param currentFile    当前文件/目录
     * @param parentId       父记录ID
     * @param parentWorkPath 父工作路径
     * @param taskId         任务ID
     * @param nodeId         节点ID
     * @param uploader       上传人
     */

    /**
     * 保存文件夹结构（分阶段处理：先处理目录，再处理文件）
     *
     * @param currentFile    当前文件/目录
     * @param parentId       父记录ID
     * @param parentWorkPath 父工作路径
     * @param taskId         任务ID
     * @param nodeId         节点ID
     * @param uploader       上传人
     */
    private void saveFolderStructure(File currentFile, String parentId, String parentWorkPath, String taskId, String nodeId, String uploader) {
        // 阶段1：处理目录结构
        processFolders(currentFile, parentId, parentWorkPath, taskId, nodeId, uploader);

        // 阶段2：处理文件
        processFiles(currentFile, parentId, parentWorkPath, taskId, nodeId, uploader);
    }

    /**
     * 阶段1：递归创建目录结构
     */
    private void processFolders(File currentFile, String parentId, String parentWorkPath,
                                String taskId, String nodeId, String uploader) {
        // 如果是文件直接返回
        if (currentFile.isFile()) return;
        // 创建或更新当前目录记录
        String currentFolderId = createOrUpdateFolderRecord(currentFile, parentId, parentWorkPath, taskId, nodeId, uploader);
        // 递归处理子目录（先处理所有子目录）
        File[] children = currentFile.listFiles();
        if (children != null) {
            for (File child : children) {
                if (child.isDirectory()) {
                    processFolders(
                            child,
                            currentFolderId,
                            buildFolderPath(parentWorkPath + currentFile.getName()),
                            taskId,
                            nodeId,
                            uploader
                    );
                }
            }
        }
    }

    /**
     * 阶段2：处理文件（此时目录结构已存在）
     */
    /**
     * 优化后的文件处理逻辑
     */
    private void processFiles(File currentFile, String parentId, String parentWorkPath,
                              String taskId, String nodeId, String uploader) {
        // 如果是文件直接处理
        if (currentFile.isFile()) {
            String actualParentId = findActualParentIdForFile(currentFile, taskId, nodeId);
            createFileRecord(currentFile, actualParentId, parentWorkPath, taskId, nodeId, uploader);
            return;
        }

        // 处理目录中的文件
        File[] children = currentFile.listFiles();
        if (children != null) {
            for (File child : children) {
                if (child.isFile()) {
                    // 关键修改：根据文件实际路径查询父目录ID
                    String actualParentId = findActualParentIdForFile(child, taskId, nodeId);
                    createFileRecord(child, actualParentId, parentWorkPath, taskId, nodeId, uploader);
                } else {
                    processFiles(
                            child,
                            parentId,
                            buildFolderPath(parentWorkPath + child.getName() + "/"), // 更新父路径
                            taskId,
                            nodeId,
                            uploader
                    );
                }
            }
        }
    }

    /**
     * 根据文件物理路径查询对应的父目录ID
     */
    private String findActualParentIdForFile(File file, String taskId, String nodeId) {
        // 获取文件的逻辑父路径
        String parentWorkPath = buildCurrentPathForParent(file);

        // 查询数据库获取目录ID
        TsFiles parentDir = tsFilesMapper.selectOne(new QueryWrapper<TsFiles>()
                .eq("task_id", taskId)
                .eq("node_id", nodeId)
                .eq("work_path", parentWorkPath)
                .eq("is_file", "FOLDER"));

        return parentDir != null ? parentDir.getParentId() : null;
    }

    /**
     * 构建文件的父目录路径（优化版）
     */
    private String buildCurrentPathForParent(File file) {
        // 获取完整逻辑路径（示例：/99/解压目录/subfolder/）
        String fullPath = buildCurrentPath("", file);

        // 确保路径中的多个 "/" 被替换成一个 "/"
        fullPath = fullPath.replaceAll("/+", "/");

        // 截取父目录部分
        if (file.isFile()) {
            // 确保在路径中有 "/" 存在，且去除文件名
            int lastSlash = fullPath.lastIndexOf('/');
            if (lastSlash != -1) {
                return fullPath.substring(0, lastSlash + 1);
            }
            // 如果路径中没有找到 "/"，说明路径仅是文件名，返回空路径
            return "/";
        }

        // 对于文件夹路径，直接返回完整路径
        return fullPath;
    }


    /**
     * 创建文件记录
     */
    private void createFileRecord(File file, String parentId, String parentWorkPath,
                                  String taskId, String nodeId, String uploader) {
        TsFiles record = buildCommonRecord(file, parentId, parentWorkPath, taskId, nodeId, uploader);
        record.setIsFile("FILE");
        record.setFileSize(String.valueOf(file.length()));
        QueryWrapper<TsFiles> query = new QueryWrapper<TsFiles>()
                .eq("node_id", nodeId)
                .eq("task_id", taskId)
                .eq("work_path", record.getWorkPath())
                .eq("file_name", file.getName())
                .eq("is_file", "FILE"); // 确保是文件类型

        TsFiles existing = tsFilesMapper.selectOne(query);
        if (existing != null) {
            existing.setUpdateTime(new Timestamp(System.currentTimeMillis()));
            tsFilesMapper.updateById(existing);
        } else {
            tsFilesMapper.insert(record);
        }
    }

    // 文件夹路径标准化（确保以/结尾）
    private String buildFolderPath(String path) {
        return path.endsWith("/") ? path : path + "/";
    }

    /**
     * 创建或更新目录记录（返回生成的目录ID）
     */
    private String createOrUpdateFolderRecord(File folder, String parentId, String parentWorkPath,
                                              String taskId, String nodeId, String uploader) {
        TsFiles record = buildCommonRecord(folder, parentId, parentWorkPath, taskId, nodeId, uploader);
        record.setIsFile("FOLDER");
        record.setFileSize("0");

        QueryWrapper<TsFiles> query = new QueryWrapper<TsFiles>()
                .eq("node_id", nodeId)
                .eq("task_id", taskId)
                .eq("work_path", record.getWorkPath())
                .eq("file_name", folder.getName())
                .eq("is_file", "FOLDER"); // 增加类型过滤

        TsFiles existing = tsFilesMapper.selectOne(query);
        if (existing != null) {
            existing.setUpdateTime(new Timestamp(System.currentTimeMillis()));
            tsFilesMapper.updateById(existing);
            return existing.getId();
        } else {
            tsFilesMapper.insert(record);
            return record.getId();
        }
    }


    /**
     * 公共记录构建方法
     */
    private TsFiles buildCommonRecord(File file, String parentId, String parentWorkPath,
                                      String taskId, String nodeId, String uploader) {
        TsFiles record = new TsFiles();
        record.setNodeId(nodeId);
        record.setTaskId(taskId);
        record.setParentId(parentId);
        record.setFileName(file.getName());
        record.setWorkPath(buildCurrentPath(parentWorkPath, file));
        record.setUploadTime(new Timestamp(System.currentTimeMillis()));
        record.setUploader(uploader);
        return record;
    }


    /**
     * 构建当前文件或文件夹的 work_path
     *
     * @return 当前文件或文件夹的 work_path
     */
    private String buildCurrentPath(String parentPath, File file) {
        // 1. 获取存储根路径（如 E:\yun）
        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);
        String rootPath = storageSourceConfig.getValue();

        // 2. 计算相对路径（统一使用正斜杠）
        String relativePath = file.getAbsolutePath()
                .replace(rootPath, "")
                .replace("\\", "/");

        // 3. 处理文件路径
        if (file.isFile()) {
            // 示例：relativePath = "99/解压目录/subfolder/file.txt" → "99/解压目录/subfolder/"
            int lastSlash = relativePath.lastIndexOf('/');
            return (lastSlash != -1 ? relativePath.substring(0, lastSlash + 1) : "");
        } else {
            // 目录路径：确保以/结尾
            int lastSlash = relativePath.lastIndexOf('/');
            return (lastSlash != -1 ? relativePath.substring(0, lastSlash + 1) : "");
        }
    }


    /**
     * 解压 ZIP 文件到临时目录
     *
     * @param zipFilePath ZIP 文件路径
     * @param baseDir     基础解压目录（如 /var/storage）
     * @return 解压后的根目录 File 对象
     */
    private File unzipToTemp(Path zipFilePath, String baseDir) throws IOException {
        // 1. 直接构建目标路径（baseDir/zipName）
        String zipName = getFileNameWithoutExtension(zipFilePath.getFileName().toString());
        Path destRoot = Paths.get(baseDir, zipName);
        Files.createDirectories(destRoot); // 确保目标目录存在

        try (ZipInputStream zis = new ZipInputStream(new FileInputStream(zipFilePath.toFile()))) {
            ZipEntry entry;
            while ((entry = zis.getNextEntry()) != null) {
                if (entry.getName().startsWith("__MACOSX/")) {
                    zis.closeEntry();
                    continue;
                }

                // 2. 直接解压到目标路径（不再嵌套临时目录）
                Path destPath = destRoot.resolve(entry.getName()).normalize();
                validatePathSafety(destPath, destRoot); // 确保路径安全

                // 3. 处理目录
                if (entry.isDirectory()) {
                    Files.createDirectories(destPath);
                }
                // 4. 处理文件
                else {
                    // 确保父目录存在
                    Files.createDirectories(destPath.getParent());

                    // 写入文件（使用缓冲提升性能）
                    try (BufferedOutputStream bos = new BufferedOutputStream(
                            Files.newOutputStream(destPath, StandardOpenOption.CREATE_NEW))) {
                        byte[] buffer = new byte[8192];
                        int len;
                        while ((len = zis.read(buffer)) > 0) {
                            bos.write(buffer, 0, len);
                        }
                    }
                }
                zis.closeEntry();
            }
        }
        return destRoot.toFile();
    }


    /**
     * 校验路径安全性（防御路径穿越攻击）
     */
    private void validatePathSafety(Path destPath, Path tempRoot) throws IOException {
        if (!destPath.startsWith(tempRoot)) {
            throw new IOException("检测到非法路径穿越: " + destPath);
        }
    }

    /**
     * 获取无扩展名的文件名
     */
    private String getFileNameWithoutExtension(String fileName) {
        int dotIndex = fileName.lastIndexOf('.');
        return (dotIndex == -1) ? fileName : fileName.substring(0, dotIndex);
    }


    /*******************************************************本地和minio对比开始****************************************************************/

    @Override
    public TsFiles compareDirectories(String localPath, String minioPath) {
        //查询本地文件路径根目录 如 E:\yun
        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);

        //查询存储空间名称根目录 如test-bucket
        QueryWrapper<StorageSourceConfig> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("name", "bucketName");
        StorageSourceConfig storageSourceConfig1 = storageSourceConfigMapper.selectOne(queryWrapper1);

        // 获取完整路径
        String fullLocalPath = localPath;
        String fullMinioPath = minioPath;
        List<FileCompareResult> results = new ArrayList<>();

        String localBasePath = storageSourceConfig.getValue() + localPath;
        // 获取文件列表
        Map<String, FileInfo> localFiles = listLocalFiles(fullLocalPath);
        TsFiles tsFiles = new TsFiles();
        try {
            // 获取minio文件列表
            AbstractBaseFileService<?> fileServiceMinio = storageSourceContext.getByStorageKey("minio");
            List<FileItemResult> fileItemListMinio = fileServiceMinio.fileLists(fullMinioPath);
            // 获取本地文件列表
            AbstractBaseFileService<?> fileServiceLocal = storageSourceContext.getByStorageKey("local");
            List<FileItemResult> fileItemListLocal = fileServiceLocal.fileLists(fullLocalPath);
            // 对比文件
            tsFiles = compareFiles(fileItemListMinio, fileItemListLocal, localBasePath, storageSourceConfig1.getValue());

        } catch (Exception e) {
            e.printStackTrace();
        }
        return tsFiles;
    }


    public String calculateMD5(InputStream inputStream) throws Exception {
        MessageDigest md = MessageDigest.getInstance("MD5");
        byte[] buffer = new byte[8192];
        int bytesRead;
        while ((bytesRead = inputStream.read(buffer)) != -1) {
            md.update(buffer, 0, bytesRead);
        }
        byte[] digest = md.digest();
        StringBuilder sb = new StringBuilder();
        for (byte b : digest) {
            sb.append(String.format("%02x", b));
        }
        return sb.toString();
    }


    public TsFiles compareFiles(List<FileItemResult> fileItemListMinio, List<FileItemResult> fileItemListLocal, String localBasePath, String bucketName) {
        TsFiles tsFiles = new TsFiles();

        // 将 MinIO 文件列表转换为 Map（文件名 -> 文件信息）
        Map<String, FileItemResult> minioFileMap = new HashMap<>();
        for (FileItemResult file : fileItemListMinio) {
            minioFileMap.put(file.getName(), file);
        }

        // 将本地文件列表转换为 Map（文件名 -> 文件信息）
        Map<String, FileItemResult> localFileMap = new HashMap<>();
        for (FileItemResult file : fileItemListLocal) {
            localFileMap.put(file.getName(), file);
        }

        // 检查本地有而 MinIO 没有的文件
        List<FileItemResult> localOnlyFiles = new ArrayList<>();
        for (FileItemResult localFile : fileItemListLocal) {
            if (!minioFileMap.containsKey(localFile.getName())) {
                localOnlyFiles.add(localFile);
            }
        }
        LOGGER.info("本地有而 MinIO 没有的文件：" + localOnlyFiles);

        // 检查 MinIO 有而本地没有的文件
        List<FileItemResult> minioOnlyFiles = new ArrayList<>();
        for (FileItemResult minioFile : fileItemListMinio) {
            if (!localFileMap.containsKey(minioFile.getName())) {
                minioOnlyFiles.add(minioFile);
            }
        }
        LOGGER.info("MinIO 有而本地没有的文件：" + minioOnlyFiles);

        // 检查文件大小不一致的文件
        List<String> sizeMismatchedFiles = new ArrayList<>();
        // 检查 MD5 不一致的文件
        //List<String> md5MismatchedFiles = new ArrayList<>();

        // 检查 MD5 不一致的文件
        List<FileItemResult> md5MismatchedFiles = new ArrayList<>();

        // 遍历 MinIO 文件
        for (FileItemResult minioFile : fileItemListMinio) {
            // 检查 MinIO 文件是否在本地存在
            if (localFileMap.containsKey(minioFile.getName())) {
                FileItemResult localFile = localFileMap.get(minioFile.getName());

                // 检查文件是否为文件夹，如果是文件夹，跳过
                File localFilePath = new File(localBasePath + localFile.getName());
                if (localFilePath.isDirectory()) {
                    LOGGER.error("路径指向的是一个文件夹，跳过该文件夹：" + localFilePath.getAbsolutePath());
                    continue; // 跳过文件夹
                }

                // 检查文件大小是否匹配
                if (minioFile.getSize() != null && localFile.getSize() != null && minioFile.getSize().longValue() != localFile.getSize().longValue()) {
                    sizeMismatchedFiles.add(minioFile.getName());
                }
                // 如果文件大小一致，进一步检查 MD5
                try {
                    // 计算本地文件的 MD5
                    String localMD5 = calculateMD5(new FileInputStream(localFilePath));
                    if (localMD5 == null || localMD5.isEmpty()) {
                        continue;
                    }

                    // 获取 MinIO 文件的 MD5
                    String minioMD5;
                    AbstractBaseFileService<?> fileServiceMinio = storageSourceContext.getByStorageKey("minio");
                    String key = StringUtils.concat(minioFile.getPath(), minioFile.getName()); // 构造文件的 key
                    if (minioFile.getSize() <= 5 * 1024 * 1024 * 1024L) { // 小于 5GB，使用 ETag
                        ObjectMetadata metadata = fileServiceMinio.getObjectMetadata(bucketName, key);
                        minioMD5 = metadata.getETag().replace("\"", ""); // 去除 ETag 的引号
                    } else { // 大于 5GB，手动计算 MD5
                        S3Object s3Object = fileServiceMinio.getObject(bucketName, key);
                        minioMD5 = calculateMD5(s3Object.getObjectContent());
                    }

                    // 比较 MD5
                    if (!minioMD5.equals(localMD5)) {
                        minioFile.setLocatMd5(localMD5);
                        minioFile.setMinioMd5(minioMD5);
                        md5MismatchedFiles.add(minioFile);
                    }
                } catch (Exception e) {
                    LOGGER.error("计算 MD5 失败：" + e.getMessage());
                }

            } else {
                // 如果 MinIO 中没有该文件，本地有的情况下，跳过该文件
                LOGGER.info("MinIO 中没有该文件，本地有文件，跳过对比: " + minioFile.getName());
            }
        }
//        tsFiles.setLocalOnlyFiles(localOnlyFiles);
//        tsFiles.setMinioOnlyFiles(minioOnlyFiles);
//        tsFiles.setMd5MismatchedFiles(md5MismatchedFiles);
        LOGGER.info("文件大小不一致的文件：" + sizeMismatchedFiles);
        LOGGER.info("文件大小一致但 MD5 不一致的文件：" + md5MismatchedFiles);
        return tsFiles;

    }


    /**
     * 递归列出本地文件
     *
     * @param path 要列出的本地目录路径
     * @return 文件信息Map，key为相对路径，value为文件信息
     */
    private Map<String, FileInfo> listLocalFiles(String path) {
        Map<String, FileInfo> files = new HashMap<>();
        File root = new File(path);
        listLocalFilesRecursive(root, root.getAbsolutePath(), files);
        return files;
    }

    /**
     * 递归列出本地文件
     *
     * @param file     当前文件/目录
     * @param basePath 基础路径（用于计算相对路径）
     * @param results  结果Map
     */
    private void listLocalFilesRecursive(File file, String basePath, Map<String, FileInfo> results) {
        String relativePath = file.getAbsolutePath().substring(basePath.length())
                .replace("\\", "/");
        if (relativePath.startsWith("/")) relativePath = relativePath.substring(1);

        if (file.isDirectory()) {
            results.put(relativePath + "/", new FileInfo());
            for (File f : file.listFiles()) {
                listLocalFilesRecursive(f, basePath, results);
            }
        } else {
            results.put(relativePath, new FileInfo());
            //results.put(relativePath, new FileInfo(calculateMD5(file), false));
        }
    }

    /**
     * 计算文件的MD5哈希值（支持大文件流式处理）
     *
     * @param file 要计算的文件对象
     * @return 32位小写十六进制MD5字符串，计算失败返回null
     */
    private String calculateMD5(File file) {
        // 前置校验
        if (!file.exists() || file.isDirectory()) {
            return null;
        }

        final int BUFFER_SIZE = 8192; // 8KB缓冲区
        try (// 自动关闭资源
             InputStream is = new FileInputStream(file);
             // 使用DigestInputStream自动处理摘要计算
             DigestInputStream dis = new DigestInputStream(is, MessageDigest.getInstance("MD5"))) {

            // 读取缓冲区（仅用于更新摘要，无需处理数据）
            byte[] buffer = new byte[BUFFER_SIZE];
            // 循环读取直到文件结束（read方法返回值被忽略，重点是通过读取更新摘要）
            while (dis.read(buffer) != -1) { /* 空循环体 */ }

            // 获取最终的摘要字节数组
            byte[] digest = dis.getMessageDigest().digest();

            // 将字节数组转换为十六进制字符串
            StringBuilder hexString = new StringBuilder();
            for (byte b : digest) {
                // 确保每个字节转换为两位十六进制
                String hex = Integer.toHexString(0xff & b);
                if (hex.length() == 1) {
                    hexString.append('0'); // 补零
                }
                hexString.append(hex);
            }
            return hexString.toString();
        } catch (NoSuchAlgorithmException e) {
            // Java标准库保证MD5存在，此处异常仅为防御性编程
            throw new RuntimeException("MD5 algorithm not found", e);
        } catch (IOException e) {
            System.err.println("Error reading file: " + file.getAbsolutePath());
            return null;
        }
    }


/*******************************************************本地和minio对比结束****************************************************************/


    /******************************************************对比结束以后开始同步***************************************************************************************/
    /**********************************
     * 用途说明: 将文件上传到备份空间
     * 参数说明  paths 路径集合
     * 参数说明  names 文件名集合
     * 参数说明  sizes 文件大小集合
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回成功或者失败
     ***********************************/
    @Override
    public Boolean uploadToBackup(String paths, String names, String sizes) {
        Boolean value = true;

        //上传人是当前登录人
//        UsernamePasswordAuthenticationToken authentication = (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
//        LoginUser loginuser = (LoginUser) authentication.getPrincipal();

        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);


        QueryWrapper<StorageSourceConfig> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("name", "bucketName");
        StorageSourceConfig storageSourceConfig1 = storageSourceConfigMapper.selectOne(queryWrapper1);

        List<String> pathsList = Arrays.asList(paths.split(","));
        List<String> nameList = Arrays.asList(names.split(","));
        List<String> sizeList = Arrays.asList(sizes.split(","));

        for (int i = 0; i < nameList.size(); i++) {
            String name = nameList.get(i).trim();
            String path = pathsList.get(i).trim();
            // 确保 path 最前面有 /
            if (!path.startsWith("/")) {
                path = "/" + path;
            }
            // 确保 path 最后面有 /
            if (!path.endsWith("/")) {
                path = path + "/";
            }
            String size = sizeList.get(i).trim();

            String localPath = storageSourceConfig.getValue() + path;//localPath是本地的路径
            String minioPath = path;//// MinIO上传目标路径
            // 使用Java NIO的Path类来构造压缩文件的完整路径，存储在localPath目录下
            Path FilePath = Paths.get(localPath, name);
            // 4. 上传压缩文件到MinIO
            File zipFile = FilePath.toFile(); // 获取本地文件


            //判断是不是文件夹  如果是 增加节点表
            File localFilePath = new File(localPath + name);
            if (localFilePath.isDirectory()) {

                String nodeName = path.replaceAll("(^/|^\\\\|/|\\\\$)", "");
                QueryWrapper<TsNodes> queryWrapperNode = new QueryWrapper<>();
                queryWrapperNode.eq("node_name", nodeName);
                TsNodes tsNodesData = tsNodesMapper.selectOne(queryWrapperNode);
                //增加节点表
                TsNodes tsnodes = new TsNodes();
                tsnodes.setParentId(tsNodesData.getNodeId());//上级节点的 节点ID
                tsnodes.setTaskId(tsNodesData.getTaskId()); //任务ID
                tsnodes.setNodeName(name);//节点名称 是文件名
                //创建人是当前登录人
                //tsnodes.setCreator(loginuser.getUsername());
                tsnodes.setCreator(null);
                //当前操作时间
                LocalDateTime now = LocalDateTime.now();
                // 转换为 Timestamp
                Timestamp currentTime = Timestamp.valueOf(now);
                tsnodes.setCreateTime(currentTime);
                //通过获取上级节点的条数 设置节点顺序
                QueryWrapper<TsNodes> queryWrapperNodeOrder = new QueryWrapper<>();
                queryWrapperNodeOrder.eq("parent_id", tsnodes.getParentId());
                int orderno = tsNodesMapper.selectCount(queryWrapperNodeOrder) + 1;
                //序号
                tsnodes.setNodeOrder(orderno);

                //判断 local或者minio有没有成功
                //新增节点的时候 创建文件夹
                NewFolderRequest newFolderRequest = new NewFolderRequest();
                newFolderRequest.setName(tsnodes.getNodeName());//新建的文件夹名称,示例值(/a/b/c)
                newFolderRequest.setPassword("");//文件夹密码, 如果文件夹需要密码才能访问，则支持请求密码,示例值(123456)
                newFolderRequest.setPath(minioPath);//请求路径,示例值(/)
                newFolderRequest.setStorageKey("minio");//存储源 key,示例值(local minio)
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
                boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());

                int valueAdded = tsNodesMapper.insert(tsnodes);
                if (valueAdded == 1) {
                    value = true;
                } else {
                    value = false;
                }


            } else {
                //如果是文件
                boolean flag1 = false;
                try {
                    AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("minio");
                    flag1 = fileService.UploadFiles(storageSourceConfig1.getValue(), minioPath + name, zipFile);
                } catch (Exception e) {
                    e.printStackTrace();
                    value = false;
                }
                //如果minio增加成功  新增表结构
                if (flag1) {
                    QueryWrapper<TsFiles> queryWrapper2 = new QueryWrapper<>();
                    queryWrapper2.eq("file_name", name);//名称
                    queryWrapper2.eq("work_path", path);//路径
                    List<TsFiles> tsFiles = tsFilesMapper.selectList(queryWrapper2);
                    //如果说没有的话 我就新增 有的话就不变
                    if (tsFiles == null || tsFiles.isEmpty()) {
                        QueryWrapper<TsFiles> queryWrapper3 = new QueryWrapper<>();
                        queryWrapper3.eq("work_path", path);//路径
                        List<TsFiles> tsFilesList = tsFilesMapper.selectList(queryWrapper3);

                        TsFiles files1 = new TsFiles();
                        files1.setTaskId(tsFilesList.get(0).getTaskId());
                        files1.setNodeId(tsFilesList.get(0).getNodeId());
                        files1.setFileName(name);
                        files1.setFileSize(size);
                        files1.setWorkPath(path);
                        files1.setBackupPath(path);
                        files1.setKeywords(null);
                        files1.setDescription(null);
                        // 设置当前时间
                        LocalDateTime now = LocalDateTime.now();
                        // 转换为 Timestamp
                        Timestamp currentTime = Timestamp.valueOf(now);
                        files1.setUploadTime(currentTime);
                        //获取登录人
                        //files1.setUploader(loginuser.getUsername());
                        files1.setUploader(null);
                        int valueAdded = tsFilesMapper.insert(files1);
                        if (valueAdded == 1) {
                            value = true;
                        } else {
                            value = false;
                        }
                    }
                }
            }
        }
        return value;
    }

    /**********************************
     * 用途说明: 从备份空间下载到工作空间
     * 参数说明  paths 路径集合
     * 参数说明  names 文件名集合
     * 参数说明  sizes 文件大小集合
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回成功或者失败
     ***********************************/
    @Override
    public Boolean downloadToLocal(String paths, String names, String sizes) {
        Boolean value = true;

        //上传人是当前登录人
//        UsernamePasswordAuthenticationToken authentication = (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
//        LoginUser loginuser = (LoginUser) authentication.getPrincipal();

        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);


        QueryWrapper<StorageSourceConfig> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("name", "bucketName");
        StorageSourceConfig storageSourceConfig1 = storageSourceConfigMapper.selectOne(queryWrapper1);

        List<String> pathsList = Arrays.asList(paths.split(","));
        List<String> nameList = Arrays.asList(names.split(","));
        List<String> sizeList = Arrays.asList(sizes.split(","));

        for (int i = 0; i < nameList.size(); i++) {
            String name = nameList.get(i).trim();
            String path = pathsList.get(i).trim();
            // 确保 path 最前面有 /
            if (!path.startsWith("/")) {
                path = "/" + path;
            }
            // 确保 path 最后面有 /
            if (!path.endsWith("/")) {
                path = path + "/";
            }
            String size = sizeList.get(i).trim();

            String localPath = storageSourceConfig.getValue() + path + name;//localPath是本地的路径
            String minioPath = path + name;//// MinIO上传目标路径

            FileOutputStream outputStream = null;   // 输出流
            S3ObjectInputStream inputStream = null; // 输入流
            S3Object s3Object = null;

            try {
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("minio");
                // 获取 MinIO 文件对象
                s3Object = fileService.getObject(storageSourceConfig1.getValue(), minioPath);
                // 获取文件输入流
                inputStream = s3Object.getObjectContent();

                // 创建本地文件
                File localFile = new File(localPath);
                if (!localFile.getParentFile().exists()) {
                    localFile.getParentFile().mkdirs(); // 创建父目录
                }

                // 将文件内容写入本地文件
                outputStream = new FileOutputStream(localFile);
                IOUtils.copy(inputStream, outputStream);
                System.out.println("文件下载成功：" + localPath);
            } catch (Exception e) {
                System.err.println("文件下载失败：" + e.getMessage());
            } finally {
                // 关闭流
                if (inputStream != null) {
                    try {
                        inputStream.close();
                    } catch (IOException e) {
                        System.err.println("关闭输入流失败：" + e.getMessage());
                    }
                }
                if (outputStream != null) {
                    try {
                        outputStream.close();
                    } catch (IOException e) {
                        System.err.println("关闭输出流失败：" + e.getMessage());
                    }
                }
                if (s3Object != null) {
                    try {
                        s3Object.close();
                    } catch (IOException e) {
                        System.err.println("关闭 S3Object 失败：" + e.getMessage());
                    }
                }
            }

            QueryWrapper<TsFiles> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.eq("file_name", name);//名称
            queryWrapper2.eq("work_path", path);//路径
            List<TsFiles> tsFiles = tsFilesMapper.selectList(queryWrapper2);
            //如果说没有的话 我就新增 有的话就不变
            if (tsFiles == null || tsFiles.isEmpty()) {
                QueryWrapper<TsFiles> queryWrapper3 = new QueryWrapper<>();
                queryWrapper3.eq("work_path", path);//路径
                List<TsFiles> tsFilesList = tsFilesMapper.selectList(queryWrapper3);

                TsFiles files1 = new TsFiles();
                files1.setTaskId(tsFilesList.get(0).getTaskId());
                files1.setNodeId(tsFilesList.get(0).getNodeId());
                files1.setFileName(name);
                files1.setFileSize(size);
                files1.setWorkPath(path);
                files1.setBackupPath(path);
                files1.setKeywords(null);
                files1.setDescription(null);
                // 设置当前时间
                LocalDateTime now = LocalDateTime.now();
                // 转换为 Timestamp
                Timestamp currentTime = Timestamp.valueOf(now);
                files1.setUploadTime(currentTime);
                //获取登录人
                //files1.setUploader(loginuser.getUsername());
                files1.setUploader(null);
                int valueAdded = tsFilesMapper.insert(files1);
                if (valueAdded == 1) {
                    value = true;
                } else {
                    value = false;
                }
            }
        }
        return value;
    }


    // 辅助方法：关闭资源时，避免抛出异常
    private void closeQuietly(AutoCloseable resource) {
        if (resource != null) {
            try {
                resource.close();
            } catch (Exception e) {
                System.err.println("关闭资源时出现异常：" + e.getMessage());
            }
        }
    }

    /*********************************************************************************************************************************************/


    /**********************************************************移动文件或者文件夹结束***********************************************************************************/
    /**
     * 移动文件或文件夹
     * 参数说明 newPath 新路径
     * 参数说明 oldpaths 原路径
     * 参数说明 ParentId 父ID
     * 参数说明 newFileName
     * 参数说明 Rename 重命名的文件名称
     * 参数说明 type 覆盖还是重命名  0 1
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String moveFileFolder(MoveCopyFileFolderRequest reques) throws IOException {
        // 1. 获取存储根路径
        StorageSourceConfig config = storageSourceConfigMapper.selectOne(
                new QueryWrapper<StorageSourceConfig>().eq("name", "filePath"));
        Path rootPath = Paths.get(config.getValue());

        String newPath = reques.getNewPath();
        String oldpaths = reques.getOldpaths();
        String parentId = reques.getParentId();
        String rename = reques.getRename();
        String type = reques.getType();
        String[] newFileNames = reques.getNewFileName().split(",");
        // 数组转集合
        List<String> newFileNameList = Arrays.asList(newFileNames);
        // 2. 构建目标父目录的物理路径（如 E:\yun\22）
        Path targetParentPhysicalPath = buildPhysicalPath(newPath, rootPath);

        // 3. 循环处理所有选中的文件/文件夹
        for (String fileName : newFileNameList) {
            // 构建原文件物理路径（如 E:\yun\11\file.txt）
            Path sourcePath = buildPhysicalPath(oldpaths + "/" + fileName, rootPath);
            if (!Files.exists(sourcePath)) {
                throw new IOException("源文件不存在: " + sourcePath);
            }

            // ==== 新增关键校验 ====
            // 1. 禁止移动到自身
            if (sourcePath.equals(targetParentPhysicalPath)) {
                throw new IllegalArgumentException("不能移动到自己所在目录");
            }

            // 2. 禁止文件夹移动到子目录
            if (Files.isDirectory(sourcePath)) {
                if (targetParentPhysicalPath.startsWith(sourcePath)) {
                    throw new IllegalArgumentException("禁止将文件夹移动到自己内部");
                }
            }

            // 生成目标文件名（处理重命名和冲突）
            String targetFileName = generateTargetFileName(targetParentPhysicalPath, fileName, rename, type);
            Path targetPath = targetParentPhysicalPath.resolve(targetFileName);

            // 3. 再次检查自移动
            if (sourcePath.equals(targetPath)) {
                throw new IllegalArgumentException("不能移动到自己当前位置");
            }

            // 执行移动操作
            moveFile(sourcePath, targetPath, type);

            // 更新数据库记录
            updateDatabase(parentId, oldpaths, fileName, newPath, targetFileName, type);
        }
        return "移动成功";
    }


    private String generateTargetFileName(Path targetParentPhysicalPath, String originalName, String rename, String type) {
        if ("1".equals(type)) {
            // 重命名模式：直接使用用户指定名称
            return rename;
        } else {
//            // 覆盖模式：处理文件名冲突（自动添加序号）
//            String baseName = originalName.lastIndexOf('.') != -1 ?
//                    originalName.substring(0, originalName.lastIndexOf('.')) : originalName;
//            String extension = originalName.lastIndexOf('.') != -1 ?
//                    originalName.substring(originalName.lastIndexOf('.')) : "";
//
//            int counter = 1;
//            String newName = originalName;
//            while (Files.exists(targetParentPhysicalPath.resolve(newName))) {
//                newName = baseName + "(" + counter + ")" + extension;
//                counter++;
//            }
            return originalName;
        }
    }

    /**
     * 构建完整的物理路径（基于数据库中的 work_path）
     */
    private Path buildPhysicalPath(String dbPath, Path root) {
        // 统一替换所有斜杠为系统分隔符（兼容不同系统）
        String systemPath = dbPath.replace("/", File.separator)
                .replace("\\", File.separator);

        // 处理根路径特殊情况（如 "/" 或空字符串）
        if (systemPath.equals(File.separator) || systemPath.isEmpty()) {
            return root;
        }

        // 修复正则表达式：使用 Pattern.quote 处理特殊字符
        String separatorRegex = Pattern.quote(File.separator);
        systemPath = systemPath.replaceAll("^" + separatorRegex + "+", "");

        try {
            Path resolvedPath = root.resolve(systemPath).normalize();

            // 二次校验防止路径越狱
            if (!resolvedPath.startsWith(root)) {
                throw new SecurityException("非法路径访问: " + dbPath);
            }
            return resolvedPath;
        } catch (InvalidPathException e) {
            throw new IllegalArgumentException("无效路径格式: " + dbPath, e);
        }
    }


    /**
     * 执行文件移动操作
     */
    private void moveFile(Path sourcePath, Path targetPath, String type) throws IOException {
        if ("0".equals(type)) {
            // 覆盖模式：强制替换目标文件
            Files.move(sourcePath, targetPath, StandardCopyOption.REPLACE_EXISTING);
        } else {
            // 重命名模式：检查目标文件是否存在
            if (Files.exists(targetPath)) {
                throw new IOException("目标文件已存在：" + targetPath);
            }
            Files.move(sourcePath, targetPath);
        }

        // 移动后检查原路径是否为空，若为空则重新创建空文件夹（可选）
        if (isPathEmpty(sourcePath.getParent())) {
            Files.createDirectories(sourcePath.getParent());
        }
    }

    /**
     * 更新数据库中的父ID和路径
     */
    private void updateDatabase(String parentId, String oldpaths, String fileName,
                                String newPath, String targetFileName, String type) {
        // 1. 构建完整路径
        String oldWorkPath = oldpaths.endsWith("/") ? oldpaths : oldpaths + "/";
        String newWorkPath = newPath.endsWith("/") ? newPath : newPath + "/";

        // 2. 查询原记录
        TsFiles fileRecord = tsFilesMapper.selectOne(
                new QueryWrapper<TsFiles>().eq("work_path", oldWorkPath)
                        .eq("file_name", fileName));
        if (fileRecord == null) {
            throw new RuntimeException("数据库记录不存在：" + oldWorkPath + fileName);
        }

        // 3. 查询目标位置是否已存在记录
        TsFiles newfileRecord = tsFilesMapper.selectOne(
                new QueryWrapper<TsFiles>().eq("work_path", newWorkPath)
                        .eq("file_name", targetFileName));

        // 4. 根据 type 处理记录
        if ("0".equals(type)) {
            // 覆盖模式
            if (newfileRecord != null) {
                // 更新目标记录
                newfileRecord.setParentId(parentId);
                newfileRecord.setWorkPath(newWorkPath);
                newfileRecord.setFileName(targetFileName);
                newfileRecord.setUpdateTime(new Timestamp(System.currentTimeMillis())); // 更新时间
                tsFilesMapper.updateById(newfileRecord);
                // 删除原记录
                tsFilesMapper.deleteById(fileRecord.getId());
            } else {
                // 插入新记录
                TsFiles newRecord = new TsFiles();
                newRecord.setNodeId(fileRecord.getNodeId());
                newRecord.setTaskId(fileRecord.getTaskId());
                newRecord.setIsFile(fileRecord.getIsFile());
                newRecord.setParentId(parentId);
                newRecord.setFileName(targetFileName);
                newRecord.setWorkPath(newWorkPath);
                newRecord.setFileSize(fileRecord.getFileSize());
                newRecord.setKeywords(fileRecord.getKeywords());
                newRecord.setDescription(fileRecord.getDescription());
                newRecord.setUploadTime(new Timestamp(System.currentTimeMillis())); // 上传时间
                newRecord.setUploader(fileRecord.getUploader()); // 上传人
                tsFilesMapper.insert(newRecord);
                // 如果是文件夹，递归删除子项
                if ("FOLDER".equals(fileRecord.getIsFile())) {
                    // 先移动子项再删除原记录
                    moveChildrenRecords(oldWorkPath, newWorkPath + targetFileName + "/", newRecord.getId(), type, fileRecord.getId());
                }
            }
            // 删除原记录
            List<String> ids = new ArrayList<>();
            ids.add(fileRecord.getId());
            deleteTsFilesByIds(ids);
        } else {
            // 重命名模式：插入新记录
            TsFiles newRecord = new TsFiles();
            newRecord.setNodeId(fileRecord.getNodeId());
            newRecord.setTaskId(fileRecord.getTaskId());
            newRecord.setIsFile(fileRecord.getIsFile());
            newRecord.setParentId(parentId);
            newRecord.setFileName(targetFileName);
            newRecord.setWorkPath(newWorkPath);
            newRecord.setFileSize(fileRecord.getFileSize());
            newRecord.setKeywords(fileRecord.getKeywords());
            newRecord.setDescription(fileRecord.getDescription());
            newRecord.setUploadTime(new Timestamp(System.currentTimeMillis())); // 上传时间
            newRecord.setUploader(fileRecord.getUploader()); // 上传人
            tsFilesMapper.insert(newRecord);
            tsFilesMapper.deleteById(fileRecord.getId());
        }
    }

    /**
     * 递归移动子项记录（覆盖模式下先移动后删除）
     */
    private void moveChildrenRecords(String oldPrefix, String newPrefix, String parentId, String type, String oldparentId) {
        //查询原来的
        List<TsFiles> children = tsFilesMapper.selectList(
                new QueryWrapper<TsFiles>().eq("parent_id", oldparentId));

        for (TsFiles child : children) {
            // 构建子项新路径
            String childNewPath = newPrefix;

            // 处理子项记录
            if ("0".equals(type)) {
                // 覆盖模式：更新或插入
                TsFiles existingChild = tsFilesMapper.selectOne(
                        new QueryWrapper<TsFiles>().eq("work_path", childNewPath)
                                .eq("file_name", child.getFileName()));

                if (existingChild != null) {
                    existingChild.setParentId(parentId);
                    existingChild.setWorkPath(childNewPath);
                    existingChild.setUpdateTime(new Timestamp(System.currentTimeMillis()));
                    tsFilesMapper.updateById(existingChild);
                } else {
                    TsFiles newChild = new TsFiles();
                    newChild.setNodeId(child.getNodeId());
                    newChild.setTaskId(child.getTaskId());
                    newChild.setIsFile(child.getIsFile());
                    newChild.setParentId(parentId);
                    newChild.setFileName(child.getFileName());
                    newChild.setWorkPath(childNewPath);
                    newChild.setFileSize(child.getFileSize());
                    newChild.setKeywords(child.getKeywords());
                    newChild.setDescription(child.getDescription());
                    newChild.setUploadTime(new Timestamp(System.currentTimeMillis()));
                    newChild.setUploader(child.getUploader());
                    tsFilesMapper.insert(newChild);
                    // 递归处理子文件夹
                    if ("FOLDER".equals(child.getIsFile())) {
                        moveChildrenRecords(oldPrefix, newPrefix + newChild.getFileName() + "/", newChild.getId(), type, child.getId());
                    }
                }
            }
        }
    }

    private String getRelativePath(String fullPath, String oldPrefix) {
        // 防御性校验：严格路径匹配
        if (!fullPath.startsWith(oldPrefix)) {
            throw new IllegalArgumentException("路径不隶属于旧目录: " + fullPath);
        }
        String relative = fullPath.substring(oldPrefix.length());
        return relative.startsWith("/") ? relative.substring(1) : relative; // 去除开头的/
    }


    /**
     * 检查路径是否为空
     */
    private boolean isPathEmpty(Path path) throws IOException {
        if (Files.isDirectory(path)) {
            try (DirectoryStream<Path> dirStream = Files.newDirectoryStream(path)) {
                return !dirStream.iterator().hasNext();
            }
        }
        return false;
    }


    /**********************************************************移动文件或者文件夹结束***********************************************************************************/


    /**********************************************************复制文件或者文件夹开始***********************************************************************************/


    @Override
    @Transactional(rollbackFor = Exception.class)
    public String copyFileFolder(MoveCopyFileFolderRequest reques) throws IOException {
        // 1. 获取存储根路径
        StorageSourceConfig config = storageSourceConfigMapper.selectOne(
                new QueryWrapper<StorageSourceConfig>().eq("name", "filePath"));
        Path rootPath = Paths.get(config.getValue());

        String newPath = reques.getNewPath();
        String oldpaths = reques.getOldpaths();
        String parentId = reques.getParentId();
        String rename = reques.getRename();
        String type = reques.getType();
        String[] newFileNames = reques.getNewFileName().split(",");
        // 数组转集合
        List<String> newFileNameList = Arrays.asList(newFileNames);

        // 2. 构建目标父目录的物理路径
        Path targetParentPhysicalPath = buildPhysicalPath(newPath, rootPath);

        // 3. 循环处理所有选中的文件/文件夹
        for (String fileName : newFileNameList) {
            // 构建原文件物理路径
            Path sourcePath = buildPhysicalPath(oldpaths + "/" + fileName, rootPath);


            if (!Files.exists(sourcePath)) {
                throw new IOException("源文件不存在: " + sourcePath);
            }

            // ==== 新增关键校验 ====
            // 1. 禁止移动到自身
            if (sourcePath.equals(targetParentPhysicalPath)) {
                throw new IllegalArgumentException("不能复制到自己所在目录");
            }

            // 2. 禁止文件夹移动到子目录
            if (Files.isDirectory(sourcePath)) {
                if (targetParentPhysicalPath.startsWith(sourcePath)) {
                    throw new IllegalArgumentException("禁止将文件夹复制到自己内部");
                }
            }

            // 生成目标文件名（处理重命名和冲突）
            String targetFileName = generateTargetFileName(targetParentPhysicalPath, fileName, rename, type);
            Path targetPath = targetParentPhysicalPath.resolve(targetFileName);

            // 3. 再次检查自移动
            if (sourcePath.equals(targetPath)) {
                throw new IllegalArgumentException("不能复制到自己当前位置");
            }
            // 执行物理复制
            copyPhysicalFile(sourcePath, targetPath, type);

            // 5. 插入新记录到数据库
            insertDatabaseRecord(parentId, oldpaths, fileName, newPath, targetFileName, type);
        }
        return "复制成功";
    }




    //========================= 核心辅助方法 =========================//

    /**
     * 复制物理文件/文件夹（带递归）
     */
    private void copyPhysicalFile(Path source, Path target, String type) throws IOException {
        if (Files.isDirectory(source)) {
            Files.createDirectories(target);
            try (DirectoryStream<Path> dirStream = Files.newDirectoryStream(source)) {
                for (Path child : dirStream) {
                    String childName = child.getFileName().toString();
                    copyPhysicalFile(child, target.resolve(childName), type);
                }
            }
        } else {
            if ("0".equals(type)) { // 覆盖模式
                Files.copy(source, target, StandardCopyOption.REPLACE_EXISTING);
            } else { // 重命名模式
                if (Files.exists(target)) throw new IOException("目标文件已存在: " + target);
                Files.copy(source, target);
            }
        }
    }

    /**
     * 插入数据库记录（主记录）
     */
    private void insertDatabaseRecord(String parentId, String oldpaths, String fileName,
                                      String newPath, String targetFileName, String type) {
        // 获取用户信息
        // 获取当前用户和时间
        UsernamePasswordAuthenticationToken authentication =
                (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();
        Timestamp currentTime = Timestamp.valueOf(LocalDateTime.now());
        //判断是不是覆盖  如果是覆盖  更新时间


        // 查询原记录
        TsFiles original = tsFilesMapper.selectOne(new QueryWrapper<TsFiles>()
                .eq("work_path", oldpaths)
                .eq("file_name", fileName)
        );
        //查询新纪录
        TsFiles neworiginal = tsFilesMapper.selectOne(new QueryWrapper<TsFiles>()
                .eq("work_path", newPath)
                .eq("file_name", targetFileName)
        );

        //如果是覆盖 并且通过新路径和名称可以查询到  那就更新时间
        if ("0".equals(type) && neworiginal != null) {

            neworiginal.setUpdateTime(currentTime);
            tsFilesMapper.updateById(neworiginal);
            if ("FOLDER".equals(original.getIsFile())) {
                copyChildrenRecords(
                        formatDbPath(oldpaths),
                        formatDbPath(newPath),
                        original.getId(),
                        neworiginal.getId(),
                        type
                );
            }

        } else {
            // 创建新记录
            TsFiles newRecord = new TsFiles();
            newRecord.setNodeId(original.getNodeId());
            newRecord.setTaskId(original.getTaskId());
            newRecord.setIsFile(original.getIsFile());
            newRecord.setParentId(parentId);
            newRecord.setFileName(targetFileName);
            newRecord.setWorkPath(formatDbPath(newPath));
            newRecord.setFileSize(original.getFileSize());
            newRecord.setKeywords(original.getKeywords());
            newRecord.setDescription(original.getDescription());
            newRecord.setUploadTime(currentTime);
            newRecord.setUploader(loginuser.getUsername());

            tsFilesMapper.insert(newRecord);
            // 递归处理子项
            if ("FOLDER".equals(original.getIsFile())) {
                copyChildrenRecords(
                        formatDbPath(oldpaths),
                        formatDbPath(newPath + newRecord.getFileName() + "/"),
                        original.getId(),
                        newRecord.getId(),
                        type
                );
            }
        }


    }

    /**
     * 递归复制子项数据库记录（关键优化点）
     */
    private void copyChildrenRecords(String oldPrefix, String newPrefix,
                                     String originalParentId, String newParentId, String type) {
        List<TsFiles> children = tsFilesMapper.selectList(
                new QueryWrapper<TsFiles>().eq("parent_id", originalParentId)
        );
        // 获取当前用户和时间
        UsernamePasswordAuthenticationToken authentication =
                (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();
        Timestamp currentTime = Timestamp.valueOf(LocalDateTime.now());

        for (TsFiles child : children) {
            // 处理当前子项
            TsFiles processedChild = processChildRecord(
                    child, newParentId, newPrefix, oldPrefix, type, loginuser, currentTime
            );

            // 递归处理子文件夹
            if ("FOLDER".equals(child.getIsFile())) {
                copyChildrenRecords(
                        oldPrefix,
                        newPrefix,
                        child.getId(),
                        processedChild.getId(), // 使用新记录的ID作为父ID
                        type
                );
            }
        }
    }

    /**
     * 处理单个子项记录
     */
    private TsFiles processChildRecord(TsFiles child, String newParentId, String newPrefix,
                                       String oldPrefix, String type, LoginUser user, Timestamp now) {
        if ("0".equals(type)) { // 覆盖模式
            TsFiles existing = tsFilesMapper.selectOne(
                    new QueryWrapper<TsFiles>()
                            .eq("work_path", buildNewWorkPath(child, oldPrefix, newPrefix))
                            .eq("file_name", child.getFileName())
            );
            if (existing != null) {
                updateExisting(existing, child, now, user);
                return existing;
            } else {
                return insertNewChild(child, newParentId, buildNewWorkPath(child, oldPrefix, newPrefix),
                        user, now, null);
            }
        } else { // 重命名模式
            return insertNewChild(child, newParentId, newPrefix,
                    user, now, child.getFileName());
        }
    }

    /**
     * 更新现有记录（覆盖模式）
     */
    private void updateExisting(TsFiles existing, TsFiles source, Timestamp time, LoginUser user) {
        existing.setFileSize(source.getFileSize());
        existing.setDescription(source.getDescription());
        existing.setUpdateTime(time);
        existing.setUploader(user.getUsername());
        tsFilesMapper.updateById(existing);
    }

    /**
     * 插入新子项记录
     */
    private TsFiles insertNewChild(TsFiles source, String parentId, String newWorkPath,
                                   LoginUser user, Timestamp time, String customName) {
        TsFiles newChild = new TsFiles();
        newChild.setNodeId(source.getNodeId());
        newChild.setTaskId(source.getTaskId());
        newChild.setIsFile(source.getIsFile());
        newChild.setParentId(parentId);
        newChild.setFileName(customName != null ? customName : source.getFileName());
        newChild.setWorkPath(newWorkPath);
        newChild.setFileSize(source.getFileSize());
        newChild.setKeywords(source.getKeywords());
        newChild.setDescription(source.getDescription());
        newChild.setUploadTime(time);
        newChild.setUploader(user.getUsername());
        tsFilesMapper.insert(newChild);
        return newChild;
    }

//========================= 工具方法 =========================//

    /**
     * 构建安全的新路径（精确替换前缀）
     */
    private String buildNewWorkPath(TsFiles child, String oldPrefix, String newPrefix) {
        // 确保旧前缀是父路径（如 /a/b/ 而不是 /a/b/file.txt）
        String childPath = child.getWorkPath();

        // 关键修复：严格匹配父路径前缀
        if (!childPath.startsWith(oldPrefix)) {
            throw new IllegalStateException("路径前缀不匹配: " + childPath);
        }

        // 计算子项相对路径（剥离旧前缀）
        String relativePath = childPath.substring(oldPrefix.length());

        // 拼接新路径（避免重复层级）
        return newPrefix + relativePath;
    }

    /**
     * 格式化数据库路径（统一以/结尾）
     */
    private String formatDbPath(String path) {
        if (path == null || path.isEmpty()) return "/";
        return path.endsWith("/") ? path : path + "/";
    }

    /**
     * 获取当前用户（封装重复逻辑）
     */
    private LoginUser getCurrentUser() {
        return (LoginUser) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
    }

    /**
     * 关键安全校验
     */
    private void validateCopySafety(Path source, Path targetParent) {
        // 禁止复制到自身目录
        if (source.equals(targetParent)) {
            throw new IllegalArgumentException("不能复制到自身目录");
        }

        // 禁止文件夹嵌套复制
        if (Files.isDirectory(source) && targetParent.startsWith(source)) {
            throw new IllegalArgumentException("禁止将文件夹复制到自身内部");
        }
    }


    /**********************************************************复制文件或者文件夹结束***********************************************************************************/

    /**********************************
     * 用途说明: 查询本地和备份空间结构树
     * 参数说明  taskId 节点ID
     * 参数说明  nodeId 任务ID
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回双树数据
     ***********************************/
    @Override
    public DualTreeResponse listLocalAndBackup(String taskId, String nodeId) {
        // 1. 查询根节点
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("node_id",nodeId);
        queryWrapper.eq("task_id",taskId);
        List<TsFiles> rootNodes = tsFilesMapper.selectList(queryWrapper);

        // 2. 构建本地树（workPath）
        List<TreeDTO> localTrees = new ArrayList<>();
        for (TsFiles rootNode : rootNodes) {
            TreeDTO localTree = buildTree(rootNode, true);
            if (localTree != null) {
                localTrees.add(localTree);
            }
        }

        // 3. 构建 Minio 树（backupPath）
        List<TreeDTO> minioTrees = new ArrayList<>();
        for (TsFiles rootNode : rootNodes) {
            TreeDTO minioTree = buildTree(rootNode, false);
            if (minioTree != null) {
                minioTrees.add(minioTree);
            }
        }

        // 4. 返回双树结构
        return new DualTreeResponse(localTrees, minioTrees);
    }

    /**
     * 递归构建树形结构
     *
     * @param tsFiles     当前节点
     * @param isLocal  是否是本地树（true: workPath, false: backupPath）
     * @return 构建好的树节点
     */
    private TreeDTO buildTree(TsFiles tsFiles, boolean isLocal) {
        // 1. 将当前节点转换为 DTO
        TreeDTO dto = convertToDTO(tsFiles, isLocal);

        // 2. 如果当前节点路径为空，则直接返回（不构建子树）
        if (dto.getPath() == null || dto.getPath().trim().isEmpty()) {
            return null;
        }

        // 3. 查询当前节点的子节点
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id",tsFiles.getId());
        List<TsFiles> children = tsFilesMapper.selectList(queryWrapper);

        if (children != null && !children.isEmpty()) {
            // 4. 递归构建子树
            for (TsFiles child : children) {
                TreeDTO childDto = buildTree(child, isLocal);
                if (childDto != null) {
                    dto.getChildren().add(childDto);
                }
            }
        }


        // 5. 返回构建好的树节点
        return dto;
    }
    /**
     * 将 FileNode 转换为 TreeDTO
     *
     * @param node     当前节点
     * @param isLocal  是否是本地树（true: workPath, false: backupPath）
     * @return 转换后的 DTO
     */
    private TreeDTO convertToDTO(TsFiles node, boolean isLocal) {
        TreeDTO dto = new TreeDTO();
        // 复制公共字段
        dto.setId(node.getId());
        dto.setNodeId(node.getNodeId());
        dto.setTaskId(node.getTaskId());
        dto.setIsFile(node.getIsFile());
        dto.setParentId(node.getParentId());
        dto.setFileName(node.getFileName());
        dto.setFileSize(node.getFileSize());
        dto.setKeywords(node.getKeywords());
        dto.setDescription(node.getDescription());
        dto.setUploadTime(node.getUploadTime());
        dto.setUploader(node.getUploader());
        dto.setUpdateTime(node.getUpdateTime());
        // 设置路径字段
        dto.setPath(isLocal ? node.getWorkPath() : node.getBackupPath());
        return dto;
    }

}
